/*******************************************************************************
*
* Licensed Materials - Property of Tivoli Systems
* 5697-B82
* (C) Copyright IBM Corp. 1997, 2001.  All rights reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*
* SAMPLE PROGRAM - NO WARRANTY EXPRESSED OR IMPLIED
*
* You are hereby licensed to use, reproduce, and distribute these sample
* programs as your needs require.  IBM does not warrant the suitability or
* integrity of these sample programs and accepts no responsibility for their
* use for your applications.  If you choose to copy and redistribute
* significant portions of these sample programs, you should preface such
* copies with this copyright notice.
*
********************************************************************************
*
* PRODUCT          (NMC)
* COMPONENT        (IHSS)
*
* MODULE_NAME      (IHSSHSTC.C)
* DESCRIPTIVE_NAME (Command exit to send a command to TME 10 NetView for OS/390)
*
********************************************************************************
*
*                       Module Level Design
*
********************************************************************************
*
* WARNING:  This sample is NOT intended to used as a command exit to be
*           executed from a console context menu.  This sample is only intended
*           to be used to initiate commands to be sent to NetView for OS/390
*           from a server workstation command prompt.  For a sample that
*           can be initiated from a console context menu, see 'ihssamce.c'.
*
*
* Function:
*
*   main()
*     - Registers the command exit and command exit procedure,
*     - Retrieves the command string and any other optional parameters from the
*       argument stack.
*     - Retrieves the value for the BINDIR environment variable.
*     - Obtains a NetView operator_id and password, in the following order:
*         1) The values from the -u and/or -p parameters.  If only one parameter
*            was specified the user will be prompted for the other.
*         2) The values from the tserver hostcmdoper utility.  This utility
*            stores the operator_id and password on disk, with the password
*            encrypted.  (Note:  If the customer changes this file, and
*            recompiles/links the ihsshstc utility, the customer is not allowed
*            to re-use the code that retrieves this password, due to
*            encryption restrictions.)
*         3) The values from the IHSSHSTC.CFG configuration file.  This is
*            a typical ascii file, that the user can edit to predefine
*            an operator_id and password.  This file is relatively insecure.
*         4) If no values are found from the above, the user will be prompted
*            for an operator_id and password.
*     - Calls IhsiSend to send the command to NetView.
*     - Printf's the command responses.
*     - Returns an error code from ihsiexit.h 'Exit Procedure Return Codes',
*       EGVE_EXIT_*.  The calling program or script can check the return
*       code and take appropriate action, including putting out a more
*       specific error message.  There are specific return codes for
*       operator not logged on, session to NetView down, etc.
*
*
* Notes:
*
*   The operator_id:
*    - Must be logged onto NetView
*    - Will be used by NetView for scope, span, and security checking.
*
*   The command string must be the first parameter, and must be enclosed
*   within double quotes, if there is more than one token.
*
*   After the command string, any of the following optional parameters
*   can be specified:
*
*     '-d'  The value after this keyword is the value of the
*           NetView domain that the command will be sent to for execution.
*
*     '-h'  The value after this keyword is the value of the TCP
*           host name or the IP address of the NetView that the command
*           will be sent to for execution.
*
*     '-u'  The value after this keyword is the NetView operator_id
*           that will execute this command.  This parameter will
*           override any pre-defined settings of the operator_id.
*
*     '-p'  The value after this keyword is the NetView password.
*           This parameter will override any pre-defined settings of
*           the password.
*
*     '-nl' A parameter indicating that No Operator Logoff request will
*           be sent to the host after the command has been processed.
*
*   If neither of the '-d' or '-h' keywords are specified, the command
*   is run on the NetView where the NETCONV session to the topology
*   server has been set up.
*
*   For example, the following the will send a command to NetView
*   domain 'CNM01':
*
*     IHSSHSTC "msg netop1 hi" -d cnm01
*
*   This program supports %RESPONSE% and %NORESPONSE% in the command
*   string to control whether the command responses are sent to the
*   NetView console, or returned to the topology server.  For example:
*
*     IHSSHSTC "%NORESPONSE% who"
*
*   will cause the "who" command to be executed at NetView; the
*   results will displayed on the NetView console of the operator used
*   for the command execution; and will not be returned to this exit.
*
*   The field wait_for_cmd_response must be filled in with a non-zero
*   value if the command responses are to be synchronously returned to
*   this exit.  Specifying %NORESPONSE% in the command string will
*   force the wait_for_cmd_response and user_data_length fields
*   to be zero.
*
*
* Compile & link instructions:
*
*   For WindowsNT, an example command to compile and link using VisualAge for C++ for NT:
*
*     set include=%include%;%BINDIR%\TDS\server\sample
*     set lib=%lib%;%BINDIR%\TDS\server\lib;
*     icc /Gm+ /Ti+ ihsshstc.c ihsp.lib ihsiexit.lib ihszras.lib ihsq.lib wsock32.lib
*
*   For OS/2, an example command to compile and link using VisualAge for C++ for OS/2:
*
*     set include=%include%;%BINDIR%\TDS\server\sample;D:\IBMCPP30\INCLUDE\sys;d:\tcpip30\include
*     set lib=%lib%;%BINDIR%\TDS\server\lib;d:\tcpip30\lib
*     icc /Gm+ /Ti+ ihsshstc.c ihsp.lib ihsiexit.lib ihszras.lib ihsq.lib so32dll.lib tcp32dll.lib
*
*   For AIX, an example command to compile and link using VisualAge for C++ for AIX:
*
*     cc_r7 -g -D__UNIX__ -D_AIX -I$BINDIR/TDS/server/sample -qcpluscmt -c ihsshstc.c
*     xlC_r7 -o ihsshstc -L$BINDIR/TDS/server/lib -lihsiexit -lihsp -legvz -lexqu -legvc -los2 -lcxxbase -lfmgda -lfmgdb ihsshstc.o
*
*
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>
#include <time.h>
#include <stdlib.h>

#ifndef __UNIX__
  #include <conio.h>
  #include <process.h>
  #define F_BINARY "b"
#endif

#ifdef __UNIX__
  #include <sys/types.h>
  #include <netdb.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <errno.h>
  #include <locale.h>
  #define F_BINARY ""
  #include <netdb.h>
  #include <sys/utsname.h>
  /* fudge Wintel-isms */
  typedef ushort BOOL;
  typedef ushort HMODULE;
  typedef char* PSZ;
  #define strnicmp strncasecmp
  #define strcmpi  strcasecmp
#endif

#ifdef __WINDOWS__
  #include <windows.h>
  #include "ihsnont.h"
  #include <winsock.h>
#endif

#ifdef __OS2__
  #include <os2.h>
  #include <types.h>
  #include <netdb.h>
  #include <sys/socket.h>
  #include <sys/un.h>
  #include <sys/ioctl.h>
  #include <netinet/in.h>
#endif

#include "ihsiexit.h"
#define STR_DECLARE_DATA                    /* Declare static string table    */
#include "ihssstri.h"                       /* String table                   */
#include "ihspmcat.h"                       /* Message catalogue              */
#include "ihszprin.h"
#include "ihsqnlsx.h"

/******************************************************************************/
/* Conditionally include macros to do encryption of passwords.  The           */
/* defining of GEM_ENCRYPTION is for IBM internal use only.  The customer     */
/* need to define DECRYPT_DATA appropriately for your environment.            */
/******************************************************************************/
#ifdef GEM_ENCRYPTION
  #include "ihssencr.h"               /* The source is NOT shipped by GEM     */
#endif

#ifndef   DECRYPT_DATA
  #define DECRYPT_DATA(data,length)
#endif

/******************************************************************************/
/* Misc defines                                                               */
/******************************************************************************/
#define TDS_SERVER_CRS_STATEMENT \
"Licensed Materials Property of IBM 5697-B83 \
(C) Copyright Tivoli Systems, an IBM Company 1999. \
All rights reserved. \
(C) Copyright IBM Corp. 1999.  All rights reserved. \
US Government Users Restricted Rights - Use duplication or \
disclosure restricted by GSA ADP Schedule Contract with IBM Corp."

#define CMD_NAME                   "Command line command"
#define THCE_PW_KEY                "IHSSHSTC_PW="    /* Password key          */
#define THCE_TCP_KEY               "TCP="            /* TCP hostname key of   */
                                                     /* NetView               */
#define THCE_DOMAIN_KEY            "DOMAIN="         /* NetView domain key    */
#define THCE_SEP                   ";"               /* Key-value separator   */
#define ARG_TCP_HOSTNAME_KEY       "-h"              /* Passed in NetView TCP */
                                                     /* hostname or IP addr   */
                                                     /* parameter             */
#define ARG_DOMAIN_KEY             "-d"              /* Passed in NetView     */
                                                     /* domain parameter      */
#define ARG_NO_LOGOFF              "-nl"             /* No Operator Logoff    */
#define ARG_DOMAIN_MAX_LEN         5                 /* Max NetView domain len*/
#define ARG_OPID_KEY               "-u"              /* Specify the operator  */
#define ARG_PASSWORD_KEY           "-p"              /* Specify the password  */
#define ARG_HELP_KEY               "-?"              /* Display syntax        */
#define TDS_ENV_VAR                "BINDIR"
#define EXIT_NAME_PREFIX           "IHSS"
#define MIN_NUMB_ARGS              1
#define MAX_NUMB_ARGS              7

#ifndef __UNIX__
#define CFG_FILE_PREFIX            "\\TDS\\server\\config\\"
#else
#define CFG_FILE_PREFIX            "/TDS/server/config/"
#endif

#define CFG_FILE_NAME              "ihsshstc.cfg"
#define ENC_FILE_NAME              "ihsshostc.cfg"
#define OPER_ID_KEY                "OPER_ID="
#define OPER_PW_KEY                "OPER_PW="
#define CMT_DEL                    "*"
#define TRUE                       1
#define FALSE                      0
#define SL_SERVER_NAME_LENGTH      128
#define SL_MAX_SERVER_NAME_LENGTH  4096
#define CSLI_IP_HEADER             -1

#ifdef __UNIX__
#define IHSS_STR_DLL_NAME          "ihssr.cat"
#else
#define IHSS_STR_DLL_NAME          "IHSSR"
#endif

#define MAX_REGISTER_ATTEMPS       100

/******************************************************************************/
/* Socket error value macros                                                  */
/******************************************************************************/
#ifdef __UNIX__
#define IHSS_PORT_GET_ERROR(__returnedError__)      \
        __returnedError__ = errno
#endif

#ifdef __OS2__
#define IHSS_PORT_GET_ERROR(__returnedError__)      \
        __returnedError__ = sock_errno()
#endif

#ifdef __WINDOWS__
#define IHSS_PORT_GET_ERROR(__returnedError__)      \
        __returnedError__ = WSAGetLastError()
#endif

/*****************************************************************************/
/* Socket error value                                                        */
/*****************************************************************************/
#ifdef __UNIX__
#define IHSS_PORT_GET_H_ERROR(__returnedError__)      \
        __returnedError__ = h_errno
#endif

#ifdef __OS2__
#define IHSS_PORT_GET_H_ERROR(__returnedError__)      \
        __returnedError__ = h_errno
#endif

#ifdef __WINDOWS__
#define IHSS_PORT_GET_H_ERROR(__returnedError__)      \
        __returnedError__ = WSAGetLastError()
#endif

/******************************************************************************/
/* Socket error values defines                                                */
/******************************************************************************/
#if ( defined(__UNIX__) || defined(__OS2__) )
  #define    PORT_SOCKET_ERROR                  -1
  #define    PORT_SOCKET_ERR_EINTR              EINTR
  #define    PORT_SOCKET_ERR_EBADF              EBADF
  #define    PORT_SOCKET_ERR_ENOTCONN           ENOTCONN
  #define    PORT_SOCKET_ERR_EWOULDBLOCK        EWOULDBLOCK
  #define    PORT_SOCKET_ERR_ECONNREFUSED       ECONNREFUSED
  #define    PORT_SOCKET_ERR_ENOBUFS            ENOBUFS
  #define    PORT_SOCKET_ERR_ETOOMANYREFS       ETOOMANYREFS
#else
  #define    PORT_SOCKET_ERROR                  SOCKET_ERROR
  #define    PORT_SOCKET_ERR_EINTR              WSAEINTR
  #define    PORT_SOCKET_ERR_EBADF              WSAEBADF
  #define    PORT_SOCKET_ERR_ENOTCONN           WSAENOTCONN
  #define    PORT_SOCKET_ERR_EWOULDBLOCK        WSAEWOULDBLOCK
  #define    PORT_SOCKET_ERR_ECONNREFUSED       WSAECONNREFUSED
  #define    PORT_SOCKET_ERR_ENOBUFS            10000
  #define    PORT_SOCKET_ERR_ETOOMANYREFS       10001
#endif

/******************************************************************************/
/* Buffer sizes                                                               */
/******************************************************************************/
#ifdef _AIX
  #define    GET_HOST_NAME_DATA_BUFFER sizeof(struct hostent_data)
  #define    GET_SERV_NAME_DATA_BUFFER sizeof(struct servent_data)
#else
  #define    GET_HOST_NAME_DATA_BUFFER 1024
  #define    GET_SERV_NAME_DATA_BUFFER 1024
#endif

/******************************************************************************/
/* Misc structures.                                                           */
/******************************************************************************/
typedef struct CSLI_IP_ADDR_DEF *CSLI_IP_ADDR_PTR,
                                 CSLI_IP_ADDR;

struct CSLI_IP_ADDR_DEF
{
  int            header;
  unsigned long  addr;
};


typedef struct CSLI_INT_IP_ADDR_DEF *CSLI_INT_IP_ADDR_PTR,
                                     CSLI_INT_IP_ADDR;

struct CSLI_INT_IP_ADDR_DEF
{
  int            header;
  int            sockfd;
};

typedef struct CSLI_LU_NAME_DEF *CSLI_LU_NAME_PTR,
                                 CSLI_LU_NAME;

struct CSLI_LU_NAME_DEF
{
  union _luName
  {
    char               lu_name[ L_IP_ADDR ];
    CSLI_IP_ADDR       ip_addr;
    CSLI_INT_IP_ADDR   int_ip_addr;
  } luName;
};


/******************************************************************************/
/*                                                                            */
/* Copyright stuff.                                                           */
/*                                                                            */
/******************************************************************************/
void
IhssPrintCopyrightStatement( void )
{
  printf("%s\n",TDS_SERVER_CRS_STATEMENT);
}


/******************************************************************************/
/*                                                                            */
/* This is the exit procedure.  The call to IhsiInitialize specifies that     */
/* this procedure should be called when the IHSSHSTC command is invoked by the*/
/* user.                                                                      */
/*                                                                            */
/* The only reason we register an exit procedure is because we are required to*/
/* do so before calling IhsiSend.  This procedure will never get called in    */
/* this example program.                                                      */
/*                                                                            */
/******************************************************************************/
unsigned int XAPI IhssHstc_exit_proc(EGVE_PARAMETERS32_PTR parms)
{
  unsigned int exit_rc;

  exit_rc = EGVE_EXIT_OK;

  return(exit_rc);
}


/******************************************************************************/
/*                                                                            */
/* Retrieves this machines hostname.                                          */
/*                                                                            */
/*    =0:  Successful                                                         */
/*    >0:  Failed, buffer to small.  Value equals the size of buffer req.     */
/*    <0:  Failed.                                                            */
/*                                                                            */
/******************************************************************************/
int
IhssHstcSocUtilLocalGetMachineName( char *pBuffer
                                  , int   bufferSize)
{
  int    rc = 0;
  int    errorValue;

  /****************************************************************************/
  /* Just get the host name for this machine and return it.                   */
  /****************************************************************************/
#if defined(SOLARIS) || defined(__LINUX_)
  struct utsname name;
  if (! uname( &name ) )
  {
     strcpy( pBuffer, name.nodename );
     pBuffer[ (bufferSize - 1) ] = '\0';
     if ( strlen(pBuffer) == (bufferSize - 1) )
     {
       rc = 1;
     }
  }
  else
  {
    IHSS_PORT_GET_ERROR(errorValue);
    printf("IhssHstcSocUtilLocalGetMachineName Loc 1, errorValue %d\n", errorValue);
    rc = -1;
  }
#else
  if ( gethostname( pBuffer, bufferSize ) != PORT_SOCKET_ERROR )
  {
     pBuffer[ (bufferSize - 1) ] = '\0';
     if ( strlen(pBuffer) == (bufferSize - 1) )
     {
       rc = 1;
     }
  }
  else
  {
    IHSS_PORT_GET_ERROR(errorValue);
    printf("IhssHstcSocUtilLocalGetMachineName Loc 2, errorValue %d\n", errorValue);
    rc = -1;
  }
#endif

  return (rc);
}


/******************************************************************************/
/*                                                                            */
/* Retrieves the hostname for this machine.                                   */
/*                                                                            */
/*  Return Codes:                                                             */
/*   ptr:  Successful                                                         */
/*         Returns a pointer to storage that contains a null terminated       */
/*         string containing the hostName.  This pointer must be freed        */
/*         by this functions caller.                                          */
/*  NULL:  Failed.                                                            */
/*                                                                            */
/******************************************************************************/
char *
IhssHstcSocUtilGetMachineName( void )
{
  int    rc = 0;
  int    retry = 1;
  int    bufferSize = SL_SERVER_NAME_LENGTH;
  char  *pBuffer;

  while (retry == 1)
  {
     if ( (pBuffer = (char *)malloc(bufferSize)) != NULL )
     {
        rc = IhssHstcSocUtilLocalGetMachineName(pBuffer,
                                            bufferSize);
        switch (rc)
        {
          case 0:
            retry = 0;
            break;
          case 1:
            if  (bufferSize < SL_MAX_SERVER_NAME_LENGTH)
              bufferSize = 2 * bufferSize;
            else
            {
              free(pBuffer);
              pBuffer = NULL;
              retry = 0;
            }
            break;
          case -1:
          default:
            free(pBuffer);
            pBuffer = NULL;
            retry = 0;
            break;
        }
     }
     else
     {
       pBuffer = NULL;
       retry = 0;
     }
  }

  return (pBuffer);
}


/******************************************************************************/
/*                                                                            */
/* Get host by name                                                           */
/*                                                                            */
/******************************************************************************/
int
IhssHstcSocUGetHostByName( struct hostent    *pMyHostent
                         , char              *pMyHostentData
                         , struct hostent   **pHost
                         , char              *hostName
                         , int                hostentDataSize )
{
  int rc = 0;

  /****************************************************************************/
  /* Intel platforms NT and OS/2                                              */
  /****************************************************************************/
  #ifndef __UNIX__
  if ( (*pHost = gethostbyname(hostName)) == NULL )
    IHSS_PORT_GET_H_ERROR(rc);
  #endif
  /****************************************************************************/
  /* AIX - need the threadsafe version                                        */
  /****************************************************************************/
  #ifdef _AIX
  memset( pMyHostentData, '\0', hostentDataSize );
  if ( gethostbyname_r( hostName
                      , pMyHostent
                      , (struct hostent_data *)pMyHostentData ) != 0 )
    IHSS_PORT_GET_H_ERROR(rc);
  #endif
  #ifdef SOLARIS
  /****************************************************************************/
  /* Solaris - need the threadsafe version                                    */
  /****************************************************************************/
  memset( pMyHostentData, '\0', hostentDataSize );
  if ( gethostbyname_r( hostName
                      , pMyHostent
                      , pMyHostentData
                      , hostentDataSize
                      , &rc ) != NULL )
    rc = 0;
  #endif
  #ifdef __LINUX__
  /****************************************************************************/
  /* Linux   - need the threadsafe version with six args                      */
  /****************************************************************************/
  memset( pMyHostentData, '\0', hostentDataSize );

  /****************************************************************************/
  /* SLES 7 (kernel 2.4.7) gives non-zero return, even though it worked.  So  */
  /* don't bother checking the return variable.                               */
  /****************************************************************************/
  gethostbyname_r( hostName, pMyHostent, pMyHostentData, hostentDataSize, pHost, &rc );

  if ( pMyHostent != NULL )
  {
    switch( pMyHostent->h_addrtype )
    {
      case AF_INET:

        if ( pMyHostent->h_addr_list != NULL )
        {
          struct in_addr       *pInetAddress;
          char                **pTemp = pMyHostent->h_addr_list;
          while ( (pInetAddress = (struct in_addr *) *pTemp++) != NULL )
          {
            rc = 0;
          }
        }
        else
        {
          fprintf( stderr, "IhssSocUGetHostByName:  FAILURE:  pmyHostent->h_addr_list is NULL\n" );
        }
        break;
      default:
        fprintf( stderr, "IhssSocUGetHostByName:  FAILURE:  pmyHostent->h_addrtype IS NOT AF_INET \n" );
        break;
    }
  }
  else
  {
    fprintf( stderr, "IhssSocUGetHostByName:  FAILURE:  pMyHostent is NULL\n" );
  }

  #endif

  return(rc);
}


/******************************************************************************/
/*                                                                            */
/* Utility to take a host name and convert it to a 32 bit address             */
/*                                                                            */
/******************************************************************************/
int
IhssHstcSocUtilGet32BitAddress( char *hostName
                              , CSLI_LU_NAME_PTR lu_name )
{
  struct hostent        *pHost;
  struct hostent         myHostent;
  #ifdef _AIX
  struct hostent_data   *pMyHostentData = NULL;
  #else
  char                  *pMyHostentData = NULL;
  #endif
  struct in_addr        *pInetAddress;
  int                    rc = 0;
  int                    errorValue;
  #ifdef _AIX
  int                    hostentSize = sizeof(struct hostent_data);
  #else
  int                    hostentSize = GET_HOST_NAME_DATA_BUFFER;
  #endif

  lu_name->luName.ip_addr.header = CSLI_IP_HEADER;

  #ifdef __UNIX__
  pHost = &myHostent;

  rc = ERANGE;

  while ( rc == ERANGE )
  {
    /**************************************************************************/
    /* UNIX needs a scratch buffer to keep this function thread safe          */
    /**************************************************************************/
    if ( pMyHostentData != NULL )
      free(pMyHostentData);

    #ifdef _AIX
    pMyHostentData = (struct hostent_data *) malloc( hostentSize );
    #else
    pMyHostentData = (char *) malloc( hostentSize );
    #endif
    if ( pMyHostentData != NULL )
    {
  #endif
      rc = IhssHstcSocUGetHostByName( &myHostent
                                    , (char *)pMyHostentData
                                    , &pHost
                                    , hostName
                                    , hostentSize );
  #ifdef __UNIX__
      if ( rc == ERANGE )
      {
        /**********************************************************************/
        /* Bag it if we have tried six times                                  */
        /**********************************************************************/
        if ( hostentSize > (GET_HOST_NAME_DATA_BUFFER * 6) )
          rc = -1;
        else
          hostentSize+= GET_HOST_NAME_DATA_BUFFER;
      }
    }
    else
      rc = -1;
  }
  #endif

  if ( rc != 0 )
  {
    if (rc == HOST_NOT_FOUND)
    {
      /************************************************************************/
      /* We must have the name in integer dotted format                       */
      /************************************************************************/
      lu_name->luName.ip_addr.addr = inet_addr(hostName);
      if ( lu_name->luName.ip_addr.addr == -1 )
      {
        IHSS_PORT_GET_ERROR(errorValue);
        printf("IhssHstcSocUtilGet32BitAddress loc 1, errorValue %d, rc %d\n", errorValue, rc);
        rc = -1;
      }
    }
    else
    {
      IHSS_PORT_GET_ERROR(errorValue);
      printf("IhssHstcSocUtilGet32BitAddress loc 2, errorValue %d, rc %d\n", errorValue, rc);
      rc = -1;
    }
  }
  else
  {
    /**************************************************************************/
    /* Check that the address we have is a TCP/IP address                     */
    /**************************************************************************/
    switch( pHost->h_addrtype )
    {
      case AF_INET:
        pInetAddress = (struct in_addr *) *pHost->h_addr_list;
        lu_name->luName.ip_addr.addr = pInetAddress->s_addr;
        break;

      default:
        printf("IhssHstcSocUtilGet32BitAddress loc 3, h_addrtype %d, rc %d\n", pHost->h_addrtype, rc);
        rc = -1;
        break;
    }
  }

  #ifdef __UNIX__
  #ifdef _AIX
  if ( pMyHostentData == NULL )
    pMyHostentData = (struct hostent_data *) malloc( sizeof(struct hostent_data) );

  if ( pMyHostentData != NULL )
  {
    memset( pMyHostentData, '\0', sizeof( *pMyHostentData ) );
    endhostent_r( pMyHostentData );
  }
  #else
  endhostent();
  #endif
  #endif

  /****************************************************************************/
  /* Free the gethostbyname buffer if we still have it                        */
  /****************************************************************************/
  if ( pMyHostentData != NULL )
    free(pMyHostentData);

  /****************************************************************************/
  /* Exit                                                                     */
  /****************************************************************************/
  return( rc );
}


/******************************************************************************/
/*                                                                            */
/* Prompt for the password.  Don't display any characters.                    */
/*                                                                            */
/******************************************************************************/
int IhssHstc_promptForPassword( char * pOperPw
                              , int    pwBuffSize )
{
#define BELL_CHAR     ((int) '\a')
#define SECRET_CHAR   ((int) '*')
#define ENTER_KEY     0xd
#define BACKSPACE_KEY 8
#define VALID_PW_KEYS "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz#@$"

  int rc = EGVE_EXIT_OK;

/******************************************************************************/
/* Unix specific implemenation                                                */
/******************************************************************************/
#ifdef __UNIX__

  /****************************************************************************/
  /* Use the offical Unix password getter.  Only gets an 8 character password */
  /* which is fine for NetView.                                               */
  /****************************************************************************/
  char * pPromptPassword = getpass( USE_A_STR(PLEASE_ENTER_PW) );

  /****************************************************************************/
  /* If the length is ok to copy                                              */
  /****************************************************************************/
  if ( strlen(pPromptPassword) < pwBuffSize )
    strcpy( pOperPw
          , pPromptPassword );

/******************************************************************************/
/* Intel specific implemenation                                               */
/******************************************************************************/
#else

  int inputLen = 0;
  int iKeyin;

  /****************************************************************************/
  /* Display the prompt message                                               */
  /****************************************************************************/
  IhszPrintf( USE_A_STR(PLEASE_ENTER_PW), 0 );

  /****************************************************************************/
  /* Get one character at a time                                              */
  /****************************************************************************/
  for(;;)
  {

    iKeyin = getch();

    /**************************************************************************/
    /* If enter key, we are done!                                             */
    /**************************************************************************/
    if (iKeyin == ENTER_KEY)
    {
      break;
    }

    /**************************************************************************/
    /* If a valid key and we haven't overrun the size                         */
    /**************************************************************************/
    if ( strchr(VALID_PW_KEYS, iKeyin) != NULL
      && inputLen+1 < pwBuffSize )
    {
      /************************************************************************/
      /* Copy the character, and output an asterisk                           */
      /************************************************************************/
      pOperPw[inputLen++] = (char) iKeyin;
      putc( SECRET_CHAR , stdout );
    }
    /**************************************************************************/
    /* Else if a backspace                                                    */
    /**************************************************************************/
    else if (iKeyin == BACKSPACE_KEY)
    {
      /************************************************************************/
      /* If something was previously entered                                  */
      /************************************************************************/
      if ( inputLen )
      {
        printf("\b \b");
        pOperPw[inputLen--] = '\0';
      }
      /************************************************************************/
      /* Else we are already at the beginning of the entered password         */
      /************************************************************************/
      else
      {
        putc( BELL_CHAR , stdout );
      }
    }
    /**************************************************************************/
    /* Else an unrecgonized character was entered, or we already have         */
    /* 8 characters.                                                          */
    /**************************************************************************/
    else
    {
      putc( BELL_CHAR , stdout );
    }

    fflush(stdout);
  }

  /****************************************************************************/
  /* Null terminate the password                                              */
  /****************************************************************************/
  pOperPw[inputLen] = '\0';

#endif

  /****************************************************************************/
  /* Ensure that the command responses are printed on the next line.          */
  /****************************************************************************/
  printf("\n\n");

  return ( rc );
}


/******************************************************************************/
/*                                                                            */
/* Get the NetView for OS/390 operator id and password from the IHSSHSTC.CFG  */
/* file, which will run the command.  If the operator id wasn't found in      */
/* the IHSSHSTC.CFG file, then prompt the user for the operator id and        */
/* password.                                                                  */
/*                                                                            */
/******************************************************************************/
int IhssHstc_getNvOperIdFromAsciiFile( char * pOperId
                                     , char * pOperPw
                                     , char * pCfgFile )
{
  int               rc = EGVE_EXIT_OK;
  FILE             *pFile;                  /* Ptr to Configuration file      */
  char              inString[120];          /* Holds a line of config file    */
  char             *pData;                  /* Ptr to data within Config file */
  char             *pTrail;                 /* Ptr to data within Config file */
  BOOL              bOperIdFound = FALSE;   /* Indicates if operId was found  */
  BOOL              bOperPwFound = FALSE;   /* Indicates if operPw was found  */
  char              promptBuffer[80];       /* Prompt buffer                  */
  char             *pMsgString;

  /****************************************************************************/
  /* Open the configuration file.  If it can't be opened, we'll prompt.       */
  /****************************************************************************/
  pFile = fopen(pCfgFile, "r");

  if ( NULL == pFile )
  {
    pMsgString = IhszTranslateString( USE_A_STR(OPEN_CONFIG_FILE) );
    fprintf( stderr
           , pMsgString
           , pCfgFile );
    IhszCleanString( USE_A_STR(OPEN_CONFIG_FILE), pMsgString );
  }

  /****************************************************************************/
  /* Read each line in the configuration file.                                */
  /****************************************************************************/
  if ( NULL != pFile )
  {
    while ( fgets(inString, sizeof(inString), pFile) != NULL )
    {
      /************************************************************************/
      /* If this is a comment line, ignore it.                                */
      /************************************************************************/
      if (strnicmp(inString, CMT_DEL, strlen(CMT_DEL)) == 0)
        continue;

      /************************************************************************/
      /* If we found one of the valid keywords.                               */
      /************************************************************************/
      if ( strstr( (const char *) inString, (const char *) OPER_ID_KEY ) != NULL
        || strstr( (const char *) inString, (const char *) OPER_PW_KEY ) != NULL )
      {
        /**********************************************************************/
        /* Skip to the end of the configuration keyword                       */
        /**********************************************************************/
        if ( strstr( (const char*) inString, (const char *) OPER_ID_KEY ) )
        {
          pData  = strstr( (const char*) inString, (const char *) OPER_ID_KEY );
          pData += strlen( OPER_ID_KEY );
        }
        else
        {
          pData  = strstr( (const char*) inString, (const char *) OPER_PW_KEY );
          pData += strlen( OPER_ID_KEY );
        }

        /**********************************************************************/
        /* Skip over any spaces between the "=" and the value.                */
        /**********************************************************************/

        while( (*pData) && isspace(*pData) )/* If we're in the string, on ws  */
            pData++;                        /* Skip over this space           */

        pTrail = pData + strlen(pData) - 1; /* Set ptr to last character      */

        /**********************************************************************/
        /* Overlay a terminating newline or carriage return with a null.      */
        /**********************************************************************/
        while( (pTrail > pData) && ( (*pTrail == '\r') || (*pTrail == '\n') ) )
        {
          *pTrail = '\0';
          pTrail--;
        }

        /**********************************************************************/
        /* Overlay any trailing spaces with a null.                           */
        /**********************************************************************/
        while ( (pTrail > pData) && isspace(*pTrail) ) /* If pData has length */
        {                                 /* greather than zero and ends in ws*/
          *pTrail = '\0';                 /* Overlay it with a null           */
          pTrail--;                       /* Backup one more character        */
        }

        /**********************************************************************/
        /* Copy the keyword value.                                            */
        /**********************************************************************/
        if ( strstr( (const char *) inString, (const char *) OPER_ID_KEY ) != NULL )

        {
          if ( bOperIdFound == FALSE )
          {
            strcpy( pOperId
                  , pData  );
          }
        }
        else
        {
          if ( bOperPwFound == FALSE )
          {
            strcpy( pOperPw
                  , pData );
          }
        }
      }
    }

    /**************************************************************************/
    /* Check if we have values for the items.                                 */
    /**************************************************************************/
    if ( strlen(pOperId) > 0 )
    {
      bOperIdFound = TRUE;
    }
    if ( strlen(pOperPw) > 0 )
    {
      bOperPwFound = TRUE;
    }

    /**************************************************************************/
    /* Close the configuration file.                                          */
    /**************************************************************************/
    fclose(pFile);
  }

  /****************************************************************************/
  /* If read the configuration file ok.                                       */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    /**************************************************************************/
    /* If the operator id key wasn't found in the configuration file.         */
    /**************************************************************************/
    if (FALSE == bOperIdFound)
    {
      pMsgString = IhszTranslateString( USE_A_STR(KEY_NOT_IN_CONFIG_FILE) );
      fprintf( stderr
             , pMsgString
             , OPER_ID_KEY
             , pCfgFile );
      IhszCleanString( USE_A_STR(KEY_NOT_IN_CONFIG_FILE), pMsgString );

      /************************************************************************/
      /* Prompt for an operator id                                            */
      /************************************************************************/
      gets( promptBuffer );

      /************************************************************************/
      /* If operator id length is bad                                         */
      /************************************************************************/
      if (strlen(promptBuffer) == 0 || strlen(promptBuffer) > L_OP_ID)
      {
        pMsgString = IhszTranslateString( USE_A_STR(OPER_ID_LEN_FR_PROMPT_INVALID) );
        fprintf( stderr
               , pMsgString
               , L_OP_ID );
        IhszCleanString( USE_A_STR(OPER_ID_LEN_FR_PROMPT_INVALID), pMsgString );
        rc = EGVE_PARAMETER_CHECK;
      }
      /************************************************************************/
      /* Else length is good                                                  */
      /************************************************************************/
      else
      {
        strcpy( pOperId
              , promptBuffer );

        /**********************************************************************/
        /* Prompt for a password                                              */
        /**********************************************************************/
        rc = IhssHstc_promptForPassword( pOperPw
                                       , L_OP_PW+1 );/* +1 for null terminator*/

        /**********************************************************************/
        /* If the password length is bad                                      */
        /**********************************************************************/
        if (strlen(pOperPw) == 0 || strlen(pOperPw) > L_OP_PW)
        {
          pMsgString = IhszTranslateString( USE_A_STR(PW_LEN_FR_PROMPT_INVALID) );
          fprintf( stderr
                 , pMsgString
                 , L_OP_PW );
          IhszCleanString( USE_A_STR(PW_LEN_FR_PROMPT_INVALID), pMsgString );
          rc = EGVE_PARAMETER_CHECK;
        }

      }

    }
    /**************************************************************************/
    /* Else operator id keyword was found in the configuration file           */
    /**************************************************************************/
    else
    {
      /************************************************************************/
      /* If the operator id length is valid                                   */
      /************************************************************************/
      if (strlen(pOperId) == 0 || strlen(pOperId) > L_OP_ID)
      {
        pMsgString = IhszTranslateString( USE_A_STR(OPER_ID_LEN_FR_CFG_INVALID) );
        fprintf( stderr
               , pMsgString
               , OPER_ID_KEY
               , L_OP_ID
               , pCfgFile );
        IhszCleanString( USE_A_STR(OPER_ID_LEN_FR_CFG_INVALID), pMsgString );
        rc = EGVE_PARAMETER_CHECK;
      }

      /************************************************************************/
      /* If the password length is valid                                      */
      /************************************************************************/
      if (strlen(pOperPw) == 0 || strlen(pOperPw) > L_OP_PW)
      {
        pMsgString = IhszTranslateString( USE_A_STR(PW_LEN_FR_CFG_INVALID) );
        fprintf( stderr
               , pMsgString
               , OPER_PW_KEY
               , L_OP_PW
               , pCfgFile );
        IhszCleanString( USE_A_STR(PW_LEN_FR_CFG_INVALID), pMsgString );
        rc = EGVE_PARAMETER_CHECK;
      }
    }

  }

  return(rc);
}


/******************************************************************************/
/*                                                                            */
/* Get the NetView for OS/390 operator id and password, which will run the    */
/* command.  This version reads the password from an encrypted cache.  If     */
/* either of the buffers passed in is not empty, its contents are preserved.  */
/*                                                                            */
/******************************************************************************/
int IhssHstc_getEncNvOperId( char * pOperId
                           , char * pOperPw
                           , char * pCfgFile )
{
  int               rc = EGVE_EXIT_OK, tmp;
  FILE             *pFile;                  /* Ptr to credential file         */
  char              opid[L_OP_ID + 1];
  char              pass[L_OP_PW + 1];
  BOOL              fileOpened = TRUE;

  memset( opid, 0, sizeof(opid) );
  memset( pass, 0, sizeof(pass) );

  /****************************************************************************/
  /* First things first: let's open the file.                                 */
  /****************************************************************************/
  pFile = fopen(pCfgFile, "r" F_BINARY);

  if ( pFile == NULL )
  {
    rc         = EGVE_PARAMETER_CHECK;
    fileOpened = FALSE;
  }

  /****************************************************************************/
  /* Read the stored user name and password.                                  */
  /****************************************************************************/
  if ( rc == EGVE_EXIT_OK )
  {
    tmp = fread(opid, L_OP_ID, 1, pFile);

    if ( tmp == L_OP_ID )
    {
      rc = EGVE_INTERNAL_FAILURE;
    }
  }

  if ( rc == EGVE_EXIT_OK )
  {
    tmp = fread(pass, L_OP_PW, 1, pFile);

    if ( tmp == L_OP_PW )
    {
      rc = EGVE_INTERNAL_FAILURE;
    }
  }

  /****************************************************************************/
  /* If we know how, decrypt the data.                                        */
  /****************************************************************************/
  if ( rc == EGVE_EXIT_OK )
  {
    DECRYPT_DATA(pass, L_OP_PW);
  }

  /****************************************************************************/
  /* Return data to the calling function.                                     */
  /****************************************************************************/
  memcpy(pOperId, opid, sizeof(opid) );
  memcpy(pOperPw, pass, sizeof(pass) );

  /****************************************************************************/
  /* Close the file if it was opened successfully.                            */
  /****************************************************************************/
  if ( fileOpened == TRUE )
  {
    fclose( pFile );
  }

  return rc;
}


/******************************************************************************/
/*                                                                            */
/* Print the Command Response strings.                                        */
/*                                                                            */
/******************************************************************************/
void IhssHstc_Cmd_Rsp(EGVE_PARAMETERS32_PTR parms)
{
  EGVE_CMD_RSP_PTR  pCmdRsp;
  unsigned short    lenCtr;
  unsigned short    prevLen;
  void             *pTmpStr;

  /****************************************************************************/
  /* Peel out the Command Response strings from user_data.  Only do this if   */
  /* the size and pointer is ok.                                              */
  /****************************************************************************/
  if ( parms->user_data_length > 0
    && NULL != parms->user_data )
  {
    /**************************************************************************/
    /* Set up ptr to point to first set of len-str pairs.                     */
    /**************************************************************************/
    pCmdRsp = (EGVE_CMD_RSP_PTR)parms->user_data;
    lenCtr  = pCmdRsp->cmdRspLen + EGVE_CMD_RSP_STR_OFFSET;

    /**************************************************************************/
    /* While the length so far is less than or equal to total length.  This   */
    /* length includes the length of the current string to be processed.      */
    /**************************************************************************/
    while (lenCtr <= parms->user_data_length)
    {
      /************************************************************************/
      /* Create a temporary string of this particular command response string.*/
      /************************************************************************/
      pTmpStr = calloc(pCmdRsp->cmdRspLen + 1, 1);

      memcpy( (unsigned char *)pTmpStr
            , pCmdRsp->cmdRspStr
            , pCmdRsp->cmdRspLen );

      /************************************************************************/
      /* Print this single command response string to stdout.                 */
      /************************************************************************/
      printf("%s \n",(unsigned char *)pTmpStr);

      free(pTmpStr);

      /************************************************************************/
      /* If we are processing the very last command response string           */
      /************************************************************************/
      if (lenCtr == parms->user_data_length)
      {
        lenCtr++;                           /* Bump ptr to get out of loop    */
      }
      /************************************************************************/
      /* Else we have more command response strings to process                */
      /************************************************************************/
      else
      {
        /**********************************************************************/
        /* Reset the pointer to the next command response string, and add the */
        /* length of the next string.                                         */
        /**********************************************************************/
        prevLen = pCmdRsp->cmdRspLen;       /* Save current cmd rsp str len   */

        pCmdRsp = (EGVE_CMD_RSP_PTR)
                  (  (unsigned char *)pCmdRsp
                    + EGVE_CMD_RSP_STR_OFFSET
                    + prevLen);

        lenCtr += pCmdRsp->cmdRspLen + EGVE_CMD_RSP_STR_OFFSET;
      }
    }                                       /* End while                      */
  }
}


/******************************************************************************/
/*                                                                            */
/* An error occurred while trying to send the EGVE_PARMS to the host          */
/* command exit IHSXTHCE.  Output an error message.                           */
/*                                                                            */
/******************************************************************************/
void IhssHstc_IhsiSend_error( int                   rc
                            , EGVE_PARAMETERS32_PTR parms
                            , char                 *pNvTcp
                            , char                 *pNvDomain )
{
  char  *pMsgString;

  pMsgString = IhszTranslateString( USE_A_STR(IHSISEND_ERROR) );
  fprintf( stderr, pMsgString, rc );
  IhszCleanString( USE_A_STR(IHSISEND_ERROR), pMsgString );

  switch(rc)
  {
    case EGVE_EXIT_NOT_LOADED:
      IhszFprintf_stderr( USE_A_STR(COMMAND_EXIT), 0 );
      break;

    case EGVE_EXIT_OUT_OF_RESOURCES:
    case EGVE_EXIT_OUT_OF_MEMORY:
      IhszFprintf_stderr( USE_A_STR(COMMAND_RESOURCES), 0 );
      break;

    case EGVE_EXIT_INVALID_CMD:
      IhszFprintf_stderr( USE_A_STR(COMMAND_CMD_STRING), 0 );
      break;

    case EGVE_EXIT_HOST_DOWN:
      IhszFprintf_stderr( USE_A_STR(COMMAND_COMM), 0 );
      break;

    case EGVE_EXIT_NO_DOMAINS:
      pMsgString = IhszTranslateString( USE_A_STR(COMMAND_DOMAIN) );
      fprintf( stderr, pMsgString, pNvTcp );
      IhszCleanString( USE_A_STR(COMMAND_DOMAIN), pMsgString );
      break;

    case EGVE_EXIT_INVALID_NV_DOMAIN:
      pMsgString = IhszTranslateString( USE_A_STR(COMMAND_INVALID_NV_DOMAIN) );
      fprintf( stderr, pMsgString, (strlen(pNvDomain) > 0)
                                     ? pNvDomain
                                     : parms->nv390_domain );
      IhszCleanString( USE_A_STR(COMMAND_INVALID_NV_DOMAIN), pMsgString );
      break;

    case EGVE_EXIT_NO_OPER:
      pMsgString = IhszTranslateString( USE_A_STR(COMMAND_OPERATOR) );
      fprintf( stderr, pMsgString, parms->op_id );
      IhszCleanString( USE_A_STR(COMMAND_OPERATOR), pMsgString );
      break;

    case EGVE_EXIT_INVALID_DEFINITION:
      IhszFprintf_stderr( USE_A_STR(COMMAND_VALIDATION), 0 );
      break;

    case EGVE_EXIT_FAIL:
      IhszFprintf_stderr( USE_A_STR(COMMAND_EXECUTE), 0 );
      break;

    case EGVE_EXIT_OPER_NOT_AUTH:
      pMsgString = IhszTranslateString( USE_A_STR(COMMAND_OPER_NOT_AUTH) );
      fprintf( stderr, pMsgString, parms->op_id );
      IhszCleanString( USE_A_STR(COMMAND_OPER_NOT_AUTH), pMsgString );
      break;

    default:
      IhszFprintf_stderr( USE_A_STR(COMMAND_EXECUTE), 0 );
      break;
  }

}


/******************************************************************************/
/*                                                                            */
/* This is the main procedure.                                                */
/*                                                                            */
/******************************************************************************/
int main( int    argc
        , char **argv )
{
  char              exit_name[L_EXIT_NAME + 1];/* Name of this command exit   */
  EGVE_PARAMETERS32 exit_parms;             /* Parameter block for Send       */
  char             *machineName;            /* IP Host Name of this machine   */
  char              IPaddr[9];              /* IP address of this machine     */
  int               rc;                     /* return code                    */
  char             *pBindir;                /* Ptr to BINDIR value            */
  char              operId[80];             /* NetView operator ID            */
  char              operPw[80];             /* NetView operator password      */
  BOOL              bOperPwAsArg   = FALSE; /* NetView password was specified */
                                            /* in args                        */
  BOOL              bHostnameAsArg = FALSE; /* NetView TCP hostname parameter */
                                            /* was entered in args            */
  BOOL              bDomainAsArg   = FALSE; /* NetView Domain name parameter  */
                                            /* was entered in args            */
  char              nvDomain[80];           /* NetView domain                 */
  char              nvTcp[80];              /* NetView TCP hostname or IP addr*/
  char              cfgFile[100];           /* Configuration file name        */
  char              encFile[100];           /* Encrypted file name            */
  short             loopCtr = 0;            /* Loop counter                   */
  short             needToTerm = FALSE;     /* EgveTerm flag                  */
  HMODULE           ihssResModHan;          /* Resource DLL Module handle     */
  short             cmdIndex = -1;          /* Which argument is the command  */
  char             *pMsgString;

  #ifdef __UNIX__
  setlocale(LC_ALL, "");
  #endif

  memset(operId     , 0x00            , sizeof(operId)          );
  memset(operPw     , 0x00            , sizeof(operPw)          );
  memset(nvDomain   , 0x00            , sizeof(nvDomain)        );
  memset(nvTcp      , 0x00            , sizeof(nvTcp)           );
  memset(&exit_parms, 0x00            , sizeof(exit_parms)      );
  memset(exit_name  , 0x00            , sizeof(exit_name)       );
  memcpy(exit_name  , EXIT_NAME_PREFIX, strlen(EXIT_NAME_PREFIX));


  #ifdef __WINDOWS__
  ExquNlsTranslateCpInitialization( 0
                                  , NULL );
  #endif

  /****************************************************************************/
  /* Load the string table.                                                   */
  /****************************************************************************/
  rc = DuipLoadStringTable( &ihssResModHan
                          , IHSS_STR_DLL_NAME
                          , ihssSTR
                          , STR_TOTAL_STRINGS );
  if (0 != rc)                              /* If string table loaded ok      */
  {
    rc = EGVE_INTERNAL_FAILURE;
    fprintf( stderr
           , "Could not load string table:  %s \n\n"
           , IHSS_STR_DLL_NAME );
  }


  /****************************************************************************/
  /* Register the exit and the exit procedure.  We make up an exit name based */
  /* on time to be reasonably unique.  If you use an exit name that is already*/
  /* in use, IhsiInitialize will fail; and thus, we will decrement the time   */
  /* by one second, until we find a correlation_number/exit_name that is not  */
  /* already registered.  This will guaurentee that the correlation number is */
  /* unique, as well as register a unique exit name.                          */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    rc = EGVE_EXIT_NOT_LOADED;

    exit_parms.correlation_id.numb = (unsigned short)time(NULL);

    while (  loopCtr      <  MAX_REGISTER_ATTEMPS
          && EGVE_EXIT_OK != rc)
    {
      exit_parms.correlation_id.numb--;

      sprintf(&exit_name[4], "%4.4X", exit_parms.correlation_id.numb);

      rc = IhsiInitialize(exit_name, IhssHstc_exit_proc);

      loopCtr++;
    }

  }

  /****************************************************************************/
  /* Read the environment variable BINDIR, to get the path name of the config */
  /* file.                                                                    */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    needToTerm = TRUE;                      /* Set flag that we need to       */
                                            /* terminate the registration of  */
                                            /* this exit                      */
    if ((pBindir = getenv(TDS_ENV_VAR)) != NULL)
    {
      strcpy(cfgFile, pBindir);
      strcat(cfgFile, CFG_FILE_PREFIX);
      strcat(cfgFile, CFG_FILE_NAME);
      strcpy(encFile, pBindir);
      strcat(encFile, CFG_FILE_PREFIX);
      strcat(encFile, ENC_FILE_NAME);
    }

    if (NULL == pBindir)
    {
      pMsgString = IhszTranslateString( USE_A_STR(NO_ENV_VAR) );
      fprintf( stderr
             , pMsgString
             , TDS_ENV_VAR );
      IhszCleanString( USE_A_STR(NO_ENV_VAR), pMsgString );
      rc = EGVE_PARAMETER_CHECK;
    }
  }

  /****************************************************************************/
  /* Else we couldn't register this instance of this Command Exit.            */
  /****************************************************************************/
  else
  {
    pMsgString = IhszTranslateString( USE_A_STR(COULD_NOT_REGISTER) );
    fprintf( stderr
           , pMsgString
           , MAX_REGISTER_ATTEMPS );
    IhszCleanString( USE_A_STR(COULD_NOT_REGISTER), pMsgString );
  }

  /****************************************************************************/
  /* Make sure the command string and other optional parameters were specified*/
  /* on the command line.  If the syntax is invalid, put out an error message */
  /* and return a parameter check return code.                                */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    if ( argc < MIN_NUMB_ARGS + 1
      || argc > MAX_NUMB_ARGS + 1 )
    {
      pMsgString = IhszTranslateString( USE_A_STR(INCORRECT_NUMB_ARGS) );
      fprintf( stderr
             , pMsgString
             , argc - 1
             , MIN_NUMB_ARGS
             , MAX_NUMB_ARGS );
      IhszCleanString( USE_A_STR(INCORRECT_NUMB_ARGS), pMsgString );

      pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
      fprintf( stderr
             , pMsgString
             , ARG_TCP_HOSTNAME_KEY
             , ARG_DOMAIN_KEY
             , ARG_OPID_KEY
             , ARG_PASSWORD_KEY
             , ARG_NO_LOGOFF );
      IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

      rc = EGVE_PARAMETER_CHECK;
    }
  }

  /****************************************************************************/
  /* Make sure that the command string is not too long to be sent to NetView. */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    if (strlen(argv[1]) == 0 || strlen(argv[1]) > EGVE_MAX_HOST_CMD_LEN)
    {
      pMsgString = IhszTranslateString( USE_A_STR(CMD_STR_LEN_INVALID) );
      fprintf( stderr
             , pMsgString
             , EGVE_MAX_HOST_CMD_LEN );
      IhszCleanString( USE_A_STR(CMD_STR_LEN_INVALID), pMsgString );

      pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
      fprintf( stderr
             , pMsgString
             , ARG_TCP_HOSTNAME_KEY
             , ARG_DOMAIN_KEY
             , ARG_OPID_KEY
             , ARG_PASSWORD_KEY
             , ARG_NO_LOGOFF );
      IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

      rc = EGVE_PARAMETER_CHECK;
    }
  }

  /****************************************************************************/
  /* Scan the argument list and validate all of the arguments.                */
  /****************************************************************************/
  for ( loopCtr = 1
      ; (EGVE_EXIT_OK == rc) && (loopCtr < argc)
      ; loopCtr++ )
  {
    /**************************************************************************/
    /* If the '-h' parameter was found                                        */
    /**************************************************************************/
    if (strcmpi(argv[loopCtr], ARG_TCP_HOSTNAME_KEY) == 0)
    {
      bHostnameAsArg = TRUE;

      if ( loopCtr < argc - 1
        && argv[loopCtr + 1] != NULL )
      {
        loopCtr++;
        strcpy( nvTcp, argv[loopCtr] );
      }
      else
      {
        pMsgString = IhszTranslateString( USE_A_STR(VALUE_FOR_PARM_MISSING) );
        fprintf( stderr
               , pMsgString
               , ARG_TCP_HOSTNAME_KEY );
        IhszCleanString( USE_A_STR(VALUE_FOR_PARM_MISSING), pMsgString );

        pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
        fprintf( stderr
               , pMsgString
               , ARG_TCP_HOSTNAME_KEY
               , ARG_DOMAIN_KEY
               , ARG_OPID_KEY
               , ARG_PASSWORD_KEY
               , ARG_NO_LOGOFF );
        IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

        rc = EGVE_PARAMETER_CHECK;

      }
    }
    else
    /**************************************************************************/
    /* If the '-d' parameter was found                                        */
    /**************************************************************************/
    if (strcmpi(argv[loopCtr], ARG_DOMAIN_KEY) == 0)
    {
      bDomainAsArg = TRUE;

      if ( loopCtr < argc - 1
        && argv[loopCtr + 1] != NULL )
      {
        if (strlen(argv[loopCtr + 1]) > ARG_DOMAIN_MAX_LEN)
        {
          pMsgString = IhszTranslateString( USE_A_STR(ARG_TOO_LONG) );
          fprintf( stderr
                 , pMsgString
                 , argv[loopCtr + 1]
                 , ARG_DOMAIN_KEY
                 , ARG_DOMAIN_MAX_LEN );
          IhszCleanString( USE_A_STR(ARG_TOO_LONG), pMsgString );

          rc = EGVE_PARAMETER_CHECK;
        }
        else
        {
          loopCtr++;
          strcpy( nvDomain, argv[loopCtr] );
          strupr( nvDomain );
        }
      }
      else
      {
        pMsgString = IhszTranslateString( USE_A_STR(VALUE_FOR_PARM_MISSING) );
        fprintf( stderr
               , pMsgString
               , ARG_DOMAIN_KEY );
        IhszCleanString( USE_A_STR(VALUE_FOR_PARM_MISSING), pMsgString );

        pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
        fprintf( stderr
               , pMsgString
               , ARG_TCP_HOSTNAME_KEY
               , ARG_DOMAIN_KEY
               , ARG_OPID_KEY
               , ARG_PASSWORD_KEY
               , ARG_NO_LOGOFF );
        IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

        rc = EGVE_PARAMETER_CHECK;
      }
    }
    else
    /**************************************************************************/
    /* If the '-u' parameter was found                                        */
    /**************************************************************************/
    if (strcmpi(argv[loopCtr], ARG_OPID_KEY) == 0)
    {
      if ( loopCtr < argc - 1
        && argv[loopCtr + 1] != NULL )
      {
        if (strlen(argv[loopCtr + 1]) > L_OP_ID)
        {
          pMsgString = IhszTranslateString( USE_A_STR(ARG_TOO_LONG) );
          fprintf( stderr
                 , pMsgString
                 , argv[loopCtr + 1]
                 , ARG_OPID_KEY
                 , L_OP_ID);
          IhszCleanString( USE_A_STR(ARG_TOO_LONG), pMsgString );

          rc = EGVE_PARAMETER_CHECK;
        }
        else
        {
          loopCtr++;
          strcpy( operId, argv[loopCtr] );
        }
      }
      else
      {
        pMsgString = IhszTranslateString( USE_A_STR(VALUE_FOR_PARM_MISSING) );
        fprintf( stderr
               , pMsgString
               , ARG_OPID_KEY );
        IhszCleanString( USE_A_STR(VALUE_FOR_PARM_MISSING), pMsgString );

        pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
        fprintf( stderr
               , pMsgString
               , ARG_TCP_HOSTNAME_KEY
               , ARG_DOMAIN_KEY
               , ARG_OPID_KEY
               , ARG_PASSWORD_KEY
               , ARG_NO_LOGOFF );
        IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

        rc = EGVE_PARAMETER_CHECK;
      }
    }
    else
    /**************************************************************************/
    /* If the '-p' parameter was found.  The value for '-p' is optional, as   */
    /* NetView allows null passwords.                                         */
    /**************************************************************************/
    if (strcmpi(argv[loopCtr], ARG_PASSWORD_KEY) == 0)
    {
      bOperPwAsArg = TRUE;

      if ( loopCtr < argc - 1
        && argv[loopCtr + 1] != NULL )
      {
        if (strlen(argv[loopCtr + 1]) > L_OP_PW)
        {
          pMsgString = IhszTranslateString( USE_A_STR(ARG_TOO_LONG) );
          fprintf( stderr
                 , pMsgString
                 , argv[loopCtr + 1]
                 , ARG_PASSWORD_KEY
                 , L_OP_PW );
          IhszCleanString( USE_A_STR(ARG_TOO_LONG), pMsgString );

          rc = EGVE_PARAMETER_CHECK;
        }
        else
        {
          loopCtr++;
          strcpy( operPw, argv[loopCtr] );
        }
      }
      else
      {
        pMsgString = IhszTranslateString( USE_A_STR(VALUE_FOR_PARM_MISSING) );
        fprintf( stderr
               , pMsgString
               , ARG_PASSWORD_KEY );
        IhszCleanString( USE_A_STR(VALUE_FOR_PARM_MISSING), pMsgString );

        pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
        fprintf( stderr
               , pMsgString
               , ARG_TCP_HOSTNAME_KEY
               , ARG_DOMAIN_KEY
               , ARG_OPID_KEY
               , ARG_PASSWORD_KEY
               , ARG_NO_LOGOFF );
        IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

        rc = EGVE_PARAMETER_CHECK;
      }
    }
    else
    /**************************************************************************/
    /* If the '-?' parameter was found                                        */
    /**************************************************************************/
    if (strcmpi(argv[loopCtr], ARG_HELP_KEY) == 0)
    {
      pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
      fprintf( stderr
             , pMsgString
             , ARG_TCP_HOSTNAME_KEY
             , ARG_DOMAIN_KEY
             , ARG_OPID_KEY
             , ARG_PASSWORD_KEY
             , ARG_NO_LOGOFF );
      IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

      rc = EGVE_PARAMETER_CHECK;
    }
    else
    /**************************************************************************/
    /*If the '-nl' No Operator Logoff parm was found                          */
    /**************************************************************************/
    if (strcmpi(argv[loopCtr], ARG_NO_LOGOFF) == 0)
    {
      /* Use reserved1[0] to indicate No Operator Logoff */
       exit_parms.reserved1[0] = 'N';
    }
    else
    /**************************************************************************/
    /* If haven't already found a parameter that may be the command, assume it*/
    /* is the command.  Remember its index for later.                         */
    /**************************************************************************/
    if (-1 == cmdIndex)
    {
      cmdIndex = loopCtr;
    }
    /**************************************************************************/
    /* Else don't recognize this parameter.                                   */
    /**************************************************************************/
    else
    {
      pMsgString = IhszTranslateString( USE_A_STR(OPT_PARM_INVALID) );
      fprintf( stderr
             , pMsgString
             , argv[loopCtr] );
      IhszCleanString( USE_A_STR(OPT_PARM_INVALID), pMsgString );

      pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
      fprintf( stderr
             , pMsgString
             , ARG_TCP_HOSTNAME_KEY
             , ARG_DOMAIN_KEY
             , ARG_OPID_KEY
             , ARG_PASSWORD_KEY
             , ARG_NO_LOGOFF );
      IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

      rc = EGVE_PARAMETER_CHECK;
    }
  }

  /****************************************************************************/
  /* Make sure that both -h and -d parameters were not entered together.      */
  /* These parameters are mutually exclusive.                                 */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    if ( TRUE == bHostnameAsArg
      && TRUE == bDomainAsArg )
    {
      pMsgString = IhszTranslateString( USE_A_STR(PARMS_ARE_MUTUALLY_EXCLUSIVE) );
      fprintf( stderr
             , pMsgString
             , ARG_TCP_HOSTNAME_KEY
             , ARG_DOMAIN_KEY );
      IhszCleanString( USE_A_STR(PARMS_ARE_MUTUALLY_EXCLUSIVE), pMsgString );

      pMsgString = IhszTranslateString( USE_A_STR(SYNTAX_IS) );
      fprintf( stderr
             , pMsgString
             , ARG_TCP_HOSTNAME_KEY
             , ARG_DOMAIN_KEY
             , ARG_OPID_KEY
             , ARG_PASSWORD_KEY
             , ARG_NO_LOGOFF );
      IhszCleanString( USE_A_STR(SYNTAX_IS), pMsgString );

      rc = EGVE_PARAMETER_CHECK;
    }
  }

  /****************************************************************************/
  /* Sanity check: If we got one and only one of 1) operator ID 2) password,  */
  /* we need to have both to continue processing, so prompt for the one that  */
  /* is missing.                                                              */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    if ( ( strlen(operId) > 0) ^ (TRUE == bOperPwAsArg) )
    {
      if ( strlen(operId) > 0 )
      {
        IhssHstc_promptForPassword( operPw, L_OP_PW + 1 );

        if ( strlen(operPw) == 0 || strlen(operPw) > L_OP_PW )
        {
          pMsgString = IhszTranslateString( USE_A_STR(PW_LEN_FR_PROMPT_INVALID) );
          fprintf( stderr
                 , pMsgString
                 , L_OP_PW);
          IhszCleanString( USE_A_STR(PW_LEN_FR_PROMPT_INVALID), pMsgString );

          rc = EGVE_PARAMETER_CHECK;
        }
      }
      else
      {
        /**********************************************************************/
        /* Display the prompt message                                         */
        /**********************************************************************/
        IhszFprintf_stderr( USE_A_STR(PLEASE_ENTER_OPID), 0 );

        /**********************************************************************/
        /* Prompt for an operator id                                          */
        /**********************************************************************/
        gets( operId );

        if ( strlen(operId) == 0 || strlen(operId) > L_OP_ID )
        {
          pMsgString = IhszTranslateString( USE_A_STR(OPER_ID_LEN_FR_PROMPT_INVALID) );
          fprintf( stderr
                 , pMsgString
                 , L_OP_ID);
          IhszCleanString( USE_A_STR(OPER_ID_LEN_FR_PROMPT_INVALID), pMsgString );

          rc = EGVE_PARAMETER_CHECK;
        }
      }
    }
  }

  /****************************************************************************/
  /* Get the local machine's IP address.  This is the IP address of the       */
  /* Graphic Data Server to send this request to.                             */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    memset(IPaddr, 0x00, sizeof(IPaddr));

    machineName = IhssHstcSocUtilGetMachineName(); /* Get IP Host Name of this*/
                                                   /* machine                 */
    if (machineName != NULL)                       /* If successful           */
    {

      if (IhssHstcSocUtilGet32BitAddress( machineName /* Get IP address       */
                                        , (CSLI_LU_NAME_PTR) IPaddr) == 0)
      {}                                    /* Successful                     */
      else                                  /* Else failure                   */
      {
        IhszFprintf_stderr( USE_A_STR(NO_IP_ADDR),0 );
        rc = EGVE_INTERNAL_FAILURE;
      }
      free(machineName);
    }
    else                                    /* Else failure                   */
    {
      IhszFprintf_stderr( USE_A_STR(NO_IP_ADDR), 0 );
      rc = EGVE_INTERNAL_FAILURE;
    }

  }

  /****************************************************************************/
  /* Read the config file, to get the NetView operator id, and NetView        */
  /* operator password.                                                       */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    #ifdef GEM_ENCRYPTION
    /**************************************************************************/
    /* If the operator id wasn't specified on the command line (keep in mind  */
    /* that the password is optional, as NetView allows nulls), read the      */
    /* encrypted credential file to get the NetView operator id and NetView   */
    /* password.                                                              */
    /**************************************************************************/
    if ( strlen(operId) == 0 )
    {
      rc = IhssHstc_getEncNvOperId( operId
                                  , operPw
                                  , encFile );
    }
    /**************************************************************************/
    /* If the file doesn't exists, (operator id and password was not pre-set  */
    /* using the 'tserver hostcmdoper' utility)                               */
    /**************************************************************************/
    if ( rc != EGVE_EXIT_OK )
    #endif

    /**************************************************************************/
    /* If the operator id isn't already retrieved (keep in mind that the      */
    /* password is optional, as NetView allows nulls), read the plaintext file*/
    /* to get the NetView operator id and password.  If the operator id wasn't*/
    /* found in the plaintext file, then prompt the user for it.              */
    /**************************************************************************/
    if ( strlen(operId) == 0 )
    {
      rc = IhssHstc_getNvOperIdFromAsciiFile( operId
                                            , operPw
                                            , cfgFile );
    }

    strupr( operId );
    strupr( operPw );
  }

  /****************************************************************************/
  /* If everything is OK, fill in the parameter block and call IhsiSend       */
  /* to send the command to NetView.                                          */
  /****************************************************************************/
  if (EGVE_EXIT_OK == rc)
  {
    exit_parms.command_name           = (char*) malloc(strlen(CMD_NAME) + 1);
    strcpy(exit_parms.command_name, CMD_NAME);                    /* Can be anything...             */

    exit_parms.command_string         = argv[cmdIndex];           /* Command string to execute      */
    exit_parms.wait_for_cmd_response  = 10;                       /* Wait 10 minutes for command    */
                                                                  /* responses.  0 will NOT return  */
                                                                  /* any command responses.  -1 will*/
                                                                  /* wait indefinitely.             */
    exit_parms.user_data_length       = 30000;                    /* Size of area for Command Rsp   */
    exit_parms.user_data              = (unsigned char *)calloc(exit_parms.user_data_length, 1);
    exit_parms.correlation_id.invoker_id = 0x8;                   /* Identifier of type of invoker  */
    strcpy (exit_parms.exit_name      , "        ");              /* Filled in by IhsiSend          */

    memset (exit_parms.op_id, ' ', L_OP_ID);                      /* NetView operator id            */
    strncpy(exit_parms.op_id, operId, strlen(operId));            /* Must be blank padded           */
    exit_parms.op_id[L_OP_ID]   = '\0';                           /*  and null terminated           */

    /************************************************************************************************/
    /* Determine length of key-value pairs to send to IHSXTHCE command exit,                        */
    /* and allocate storage.                                                                        */
    /************************************************************************************************/
    exit_parms.user_ctrl_data_length  =   strlen(THCE_PW_KEY)     /* Calculate length of:  PW key   */
                                        + strlen(operPw)          /* PW value                       */
                                        + 1;                      /* Null terminator                */
    if (strlen(nvTcp) > 0)
    {
      exit_parms.user_ctrl_data_length  +=strlen(THCE_SEP)        /* Key-value separator            */
                                        + strlen(THCE_TCP_KEY)    /* NetView TCP hostname key       */
                                        + strlen(nvTcp);          /* NetView TCP hostname value     */
    }
    else
    if (strlen(nvDomain) > 0)
    {
      exit_parms.user_ctrl_data_length  +=strlen(THCE_SEP)        /* Key-value separator            */
                                        + strlen(THCE_DOMAIN_KEY) /* Domain key                     */
                                        + strlen(nvDomain);       /* Domain value                   */
    }
    exit_parms.user_ctrl_data         = (unsigned char *)malloc(exit_parms.user_ctrl_data_length);

    /************************************************************************************************/
    /* Build key-value pairs to send to IHSXTHCE command exit.                                      */
    /************************************************************************************************/
    strcpy((char*)exit_parms.user_ctrl_data, THCE_PW_KEY);        /* Build PW key                   */
    strcat((char*)exit_parms.user_ctrl_data, operPw);             /* Concatenate PW value           */
    if (strlen(nvTcp) > 0)
    {
      strcat((char*)exit_parms.user_ctrl_data, THCE_SEP);         /* Concatenate key-value separator*/
      strcat((char*)exit_parms.user_ctrl_data, THCE_TCP_KEY);     /* Concatenate TCP hostname key   */
      strcat((char*)exit_parms.user_ctrl_data, nvTcp);            /* Concatenate TCP hostname value */
    }
    else
    if (strlen(nvDomain) > 0)
    {
      strcat((char*)exit_parms.user_ctrl_data, THCE_SEP);         /* Concatenate key-value separator*/
      strcat((char*)exit_parms.user_ctrl_data, THCE_DOMAIN_KEY);  /* Concatenate domain key         */
      strcat((char*)exit_parms.user_ctrl_data, nvDomain);         /* Concatenate domain value       */
    }

    strcpy (exit_parms.client_ip_addr  , IPaddr);                 /* Local IP address               */
    strcpy (exit_parms.server_ip_addr  , IPaddr);                 /* Local IP address               */

    #ifndef __UNIX__
    exit_parms.client_handle          = 0x80000000;               /* Use a dummy but valid client ID*/
    #else
    exit_parms.client_handle          = 0x00000008;
    #endif

    /**************************************************************************/
    /* Drive the IHSXTHCE command exit to send the command to the mainframe   */
    /**************************************************************************/
    rc = IhsiSend( exit_name                /* Source Exit name               */
                 , "IHSXTHCE"               /* Dest Exit name                 */
                 , IPaddr                   /* Dest IP address                */
                 , &exit_parms);            /* Exit Paramters                 */

    /**************************************************************************/
    /* If successful, handle the Command Responses.                           */
    /**************************************************************************/
    if (EGVE_EXIT_OK == rc)
      IhssHstc_Cmd_Rsp(&exit_parms);

    /**************************************************************************/
    /* Else output an error message                                           */
    /**************************************************************************/
    else
      IhssHstc_IhsiSend_error( rc
                             , &exit_parms
                             , nvTcp
                             , nvDomain );

    /**************************************************************************/
    /* Perform final cleanup                                                  */
    /**************************************************************************/
    free(exit_parms.command_name);                   /* Free malloc'ed storage*/
    exit_parms.command_name = NULL;

    free(exit_parms.user_data);                      /* Free malloc'ed storage*/
    exit_parms.user_data             = NULL;
    exit_parms.user_data_length      = 0;

    free(exit_parms.user_ctrl_data);                 /* Free malloc'ed storage*/
    exit_parms.user_ctrl_data        = NULL;
    exit_parms.user_ctrl_data_length = 0;

  }

  /****************************************************************************/
  /* If we successfully registered, then we need to terminate the registration*/
  /****************************************************************************/
  if (TRUE == needToTerm)
    IhsiTerminate(exit_name);                        /* De-register the exit  */

  #ifdef __WINDOWS__
  ExquNlsTranslateCpTerminate();
  #endif

  return(rc);
}
